class Editor.EditorModule.DockModule.NewElementBoard extends Editor.EditorModule.DockModule.Board
  
  initialize: (options) ->
    super(options)
    @panel = @parent.dock().editor().panel
  
class Editor.EditorModule.DockModule.TabModule.NewElementBoardLabel extends Editor.EditorModule.DockModule.TabModule.BoardLabel
  
  initialize: (options) ->
    super(options)
  
  render: () ->
    super()
    this.$el.attr('data-z', 0)
    this.$el.removeClass('ui-corner-top')
    this.$el.addClass('ui-corner-left')
    this
  
  face: () ->
    icon = new Pettanr.Image.SymbolImg({attr: {src: '/images/new.gif'}})
    linked_elements_tab = new Tag.A({
      attr: {href: '#' + @board().body.dom_id() },
      content: icon.render().el
    })
    this.$el.append(linked_elements_tab.render().el)
  
  add_label: () ->
    this.$el.attr('id', @dom_id())
  
  z: () ->
    parseInt(this.$el.attr('data-z'))
  
class Editor.EditorModule.DockModule.TabModule.NewElementBoardBody extends Editor.EditorModule.DockModule.TabModule.BoardBody
  className: 'new-element-board-body'
  
  initialize: (options) ->
    super(options)
  
  render: () ->
    this.$el.attr('id', @dom_id())
    @launchers = new Editor.EditorModule.DockModule.NewElementLaunchers({
      parent: this, element_models: @element_models()
    })
    @listenTo(@launchers, 'pick', @pick)
    @listenTo(@launchers, 'dialog:open', @open_dialog)
    @listenTo(@launchers, 'dialog:close', @close_dialog)
    this.$el.append(@launchers.render().el)
    this
  
  panel: () ->
    @parent.panel
  
  element_models: () ->
    @panel().my_class().child_models()
  
  pick: (new_item) ->
    @trigger('pick', new_item)
  
  open_dialog: (dialog) ->
    @trigger('dialog:open', dialog)
  
  close_dialog: (dialog) ->
    @trigger('dialog:close', dialog)
  
class Editor.EditorModule.DockModule.NewElementLaunchers extends Backbone.View
  tagName: 'div'
  className: 'new-element-buttons'
  
  initialize: (options) ->
    super(options)
    @parent = options.parent
    @element_models = options.element_models
  
  render: () ->
    _.each @element_models, (model) =>
      launcher = new Pettanr.Views[model.singular()].NewElementLauncher({
        parent: this, model: model
      })
      @listenTo(launcher, 'pick', @pick)
      @listenTo(launcher, 'dialog:open', @open_dialog)
      @listenTo(launcher, 'dialog:close', @close_dialog)
      this.$el.append(launcher.render().el)
    this
  
  pick: (new_item) ->
    @trigger('pick', new_item)
  
  open_dialog: (dialog) ->
    @trigger('dialog:open', dialog)
  
  close_dialog: (dialog) ->
    @trigger('dialog:close', dialog)
  
  dock: () ->
    @parent.dock()
  
class Editor.EditorModule.DockModule.NewElementLauncher extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    super(options)
    @parent = options.parent
    @model = options.model
  
  render: () ->
    this.$el.html('')
    header = new Editor.EditorModule.DockModule.NewElementLauncherHeader({model: @model})
    this.$el.append(header.render().el)
    this
  
  get_action: (params) ->
    controller = Manifest.manifest().controllers[params['controller']]
    controller.actions[params['action']]
  
  add_links: () ->
    _.each @links_conf, (conf, name) =>
      @add_link(name, conf.caption, @get_action(conf.params).url(conf.params))
  
  add_link: (name, caption, url) ->
    link = new Tag.A({
      attr: {href: '/' + url},
      content: caption
    })
    @listenTo(link, 'click', this['click_' + name])
    this.$el.append(link.render().el)
  
  create_dialog: () ->
    dialog = new Editor.PanelEditor.Dialog({
      parent: this
    })
    this.$el.append(dialog.render().el)
    _this = this
    dialog.$el.dialog({
      autoOpen: false,
      width: 750, 
      height: 560,
      close: (ui, event) ->
        _this.close_dialog(dialog)
    })
    dialog
  
  click_default: (params) ->
    dialog = @create_dialog()
    picker = new Pettanr.Views[@model.singular()].NewElement({
      parent: this, target_model: @model
    })
    @listenTo(picker, 'pick', @pick)
    @listenTo(picker, 'dialog:open', @open_dialog)
    @listenTo(picker, 'dialog:close', @close_dialog)
    picker.start(dialog, params)
  
  pick: (new_item) ->
    @trigger('pick', new_item)
  
  open_dialog: (dialog) ->
    @trigger('dialog:open', dialog)
  
  close_dialog: (dialog) ->
    @trigger('dialog:close', dialog)
  
  dock: () ->
    @parent.dock()
  
class Editor.EditorModule.DockModule.NewElementLauncherHeader extends Backbone.View
  tagName: 'h3'
  
  initialize: (options) ->
    super(options)
    @model = options.model
  
  render: () ->
    this.$el.html('')
    icon = new Pettanr.Image.Icon({item: @model, half: true})
    caption = Pettanr.AppHelper.t_m(@model.singular())
    this.$el.append(icon.render().el)
    this.$el.append(caption)
    this
  
