class Editor.EditorModule.DockModule.ScenarioBay extends Editor.EditorModule.DockModule.SimpleBay
  
  initialize: (options) ->
    super(options)
  
  render: () ->
    @body.render()
  
  add_element: (element) ->
    @body.add_element(element)
  
  init_tabs: () ->
    list = @root_item().scenario_elements()
    _.each list, (element) =>
      @add_element(element)
  
  save_data: () ->
    @body.form.save_data()
  
  length: () ->
    @body.length()
  
class Editor.EditorModule.DockModule.TabModule.ScenarioBayLabel extends Editor.EditorModule.DockModule.TabModule.BayLabel
  
class Editor.EditorModule.DockModule.TabModule.ScenarioBayBody extends Editor.EditorModule.DockModule.TabModule.BayBody
  
  initialize: (options) ->
    super(options)
    @elements = new Editor.EditorModule.DockModule.ScenarioBay.Elements({
      parent: this, 
    })
    @listenTo(@elements, 'sorted', @sorted)
  
  add_element: (element) ->
    @elements.add_element(element)
  
  render: () ->
    super()
    this.$el.append(@elements.render().el)
    @delegateEvents({'activate': 'activate'})
    this
  
  activate: () ->
    @elements.refresh()
  
  sorted: () ->
    # data-t copy to model
    _.each @elements.element_lis, (element_li) =>
      t = parseInt(element_li.$el.attr('data-t'))
      element_li.element.set({
        t: t
      }, {silent: true})
      element_li.element.trigger('sort')
  
  panel_body: () ->
    @dock().editor().body
  
  save_data: () ->
    attrs = {}
    _.each @tabs, (tab) ->
      return if not tab.element  # without new element tab
      form = tab.body.form
      name = form.item.table_name() + '_attributes'
      attrs[name] ||= []
      attrs[name].push(form.save_data())
    attrs
  
  length: () ->
    @elements.element_lis.length
  
class Editor.EditorModule.DockModule.ScenarioBay.Elements extends Backbone.View
  tagName: 'ul'
  className: 'scenarios_elements'
  
  initialize: (options) ->
    super(options)
    @parent = options.parent
    @element_lis = []
  
  render: () ->
    this.$el.html('')
    _.each @scenario_elements(), (element_li) =>
      this.$el.append(element_li.render().$el)
    _this = this
    this.$el.sortable {
      update: (event, ui) ->
        t = 0
        $('.t-label', $(@)).map ->
          $(@).attr('data-t', t)
          t++
        _this.trigger('sorted')
    }
    this
  
  refresh: () ->
    _.each @element_lis, (element_li) =>
      element_li.refresh()
  
  scenario_elements: () ->
    res = []
    _.each @element_lis, (element_li) =>
      res[element_li.t()] = element_li
    res
  
  add_element: (element) ->
    e = new Editor.EditorModule.DockModule.ScenarioBay.Element({
      parent: this, 
      element: element
    })
    @element_lis.push(e) # insert array
    @render()
    e
  
  sorted: () ->
    # data-t copy to model
    @trigger('sorted')
  
class Editor.EditorModule.DockModule.ScenarioBay.Element extends Backbone.View
  tagName: 'li'
  className: 't-label'
  
  initialize: (options) ->
    super(options)
    @parent = options.parent
    @element = options.element
  
  render: () ->
    this.$el.html('')
    this.$el.attr('data-t', @t())
    @label = new Editor.EditorModule.DockModule.ScenarioBay.ElementLabel({
      parent: this, 
    })
    @body = new Editor.EditorModule.DockModule.ScenarioBay.ElementBody({
      parent: this, 
    })
    this.$el.append(@label.render().$el)
    this.$el.append(@body.render().$el)
    rb = new Tag.RowBreak()
    this.$el.append(rb.render().el)
    this
  
  refresh: () ->
    @body.form.refresh()
  
  t: () ->
    @element.get('t')
  
  dom_id: () ->
    super() + '-' + @element.get('id')  # use tab index
  
class Editor.EditorModule.DockModule.ScenarioBay.ElementLabel extends Backbone.View
  className: 'scenario-label'
  
  initialize: (options) ->
    super(options)
    @parent = options.parent
  
  render: () ->
    this.$el.append(@element().element_face().render().el)
    _this = this
    this.$el.mouseover -> 
      _this.element().trigger('active')
    this.$el.mouseout -> 
      _this.element().trigger('inactive')
    this
  
  element: () ->
    @parent.element
  
  elements: () ->
    @parent.parent
  
  bay: () ->
    @elements().parent
  
  dock: () ->
    @bay().dock()
  
class Editor.EditorModule.DockModule.ScenarioBay.ElementBody extends Backbone.View
  
  initialize: (options) ->
    super(options)
    @parent = options.parent
    @form = Locmare.Form.factory({
      form_name: @element().item_name(), 
      use_name: 'scenario' 
      item: @element(), 
      element_form: true, 
      submit: null, 
    })
  
  render: () ->
    this.$el.html(@form.render().el)
    this
  
  element: () ->
    @parent.element
  
  elements: () ->
    @parent.parent
  
  bay: () ->
    @elements().parent
  
  dock: () ->
    @bay().dock()
  
  mounted: () ->
    true
  
  element: () ->
    @parent.element
  
class Editor.EditorModule.DockModule.NewBay extends Editor.EditorModule.DockModule.BoardBay
  

class Editor.EditorModule.DockModule.TabModule.NewBayLabel extends Editor.EditorModule.DockModule.TabModule.BayLabel
  

class Editor.EditorModule.DockModule.TabModule.NewBayBody extends Editor.EditorModule.DockModule.TabModule.BayBody
  
  initialize: (options) ->
    super(options)
  
class Editor.EditorModule.DockModule.NewBoard extends Editor.EditorModule.DockModule.Board
  
class Editor.EditorModule.DockModule.TabModule.NewBoardLabel extends Editor.EditorModule.DockModule.TabModule.BoardLabel
  
class Editor.EditorModule.DockModule.TabModule.NewBoardBody extends Editor.EditorModule.DockModule.TabModule.BoardBody
  
  initialize: (options) ->
    super(options)
  
