class Pettanr.Views.Scroll.PlayModule
class Pettanr.Views.Scroll.PlayModule.Header extends Pettanr.Views.Show.Header
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Scroll.PlayModule.Owner extends Pettanr.Views.Show.Owner
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Scroll.PlayModule.Footer extends Backbone.View
  tagName: 'table'
  className: 'no-border'
  
  initialize: (options) ->
    @panel = options.panel
    @scroll_panel = options.scroll_panel
  
  render: () ->
    this.$el.html('')
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@panel.get('updated_at')))
    #
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @scroll_panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@scroll_panel.get('updated_at')))
    #
    if @panel.is_own()
      this.$el.append('')
    this
  
class Pettanr.Views.Scroll.PlayModule.Credits extends Backbone.View
  tagName: 'div'
  className: 'credits'
  
  initialize: (options) ->
    @parent = options.parent
    @licensed_pictures = {}
  
  render: () ->
    this.$el.html('')
    this
  
  push: (retrievers) ->
    _.each retrievers, (retriever) =>
      @listenTo(retriever, 'retrieve', @retrieve_picture)
      retriever.retrieve()
  
  retrieve_picture: (picture) ->
    pid = picture.get('id')
    return if @licensed_pictures[pid]
    @licensed_pictures[pid] = picture
    credit = picture.credit_view()
    this.$el.append(credit.render().el)
  
class Pettanr.Views.Scroll.PlayModule.Panel extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @scroll_panel = options.scroll_panel
  
  clear: () ->
    this.$el.html('')
    retriever = @scroll_panel.panel()
    @listenTo(retriever, 'retrieve', @retrieve_panel)
    retriever.retrieve()
    this
  
  retrieve_panel: (@panel) ->
    @render()
  
  render: () ->
    this.$el.html('')
    @panel = @panel.with_elements() # retake panel for 'with_elements' mode
    @panel.fetch({cache: true}).done =>
      @panel.attributes = @panel.replaced_attributes()
      body = new Pettanr.Views.Panel.Body({
        panel: @panel,
        spot: null
      })
      footer = new Pettanr.Views.Panel.Footer({
        scroll_panel: @scroll_panel,
        panel: @panel,
      })
      @listenTo(footer, 'click:panel', @click_show)
      @listenTo(footer, 'click:author', @click_author)
      @listenTo(footer, 'click:edit', @click_edit)
      this.$el.append(body.render().el)
      this.$el.append(footer.render().el)
      @trigger('ready', @panel)
    this
  
  click_show: () ->
    @trigger('http_get', @panel.show_url())
  
  click_author: () ->
    @trigger('http_get', @panel.author().show_url())
  
  click_edit: () ->
    @trigger('http_get', @panel.edit_url())
  
class Pettanr.Views.Scroll.PlayModule.Panels extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @items = options.items
    @credits = options.credits
  
  render: () ->
    this.$el.html('')
    _.each @items, (scroll_panel) =>
      return if not scroll_panel.has_panel()
      panel_view = new Pettanr.Views.Scroll.PlayModule.Panel({
        scroll_panel: scroll_panel,
      })
      @listenTo(panel_view, 'ready', @ready)
      this.$el.append(panel_view.clear().el)
    this
  
  ready: (panel) ->
    this.credits.push(panel.licensed_pictures())
  
class Pettanr.Views.Scroll.PlayModule.Body extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @list = options.list
    @items = @list.items()
    if @items.length < 1
      @render_empty()
    else
      @render()
  
  render_empty: () ->
    this.$el.html('')
    this.$el.append((new Tag.H2({content: I18n.t('scrolls.play.empty')})).render().el)
    this
  
  render: () ->
    this.$el.html('')
    # paginate(@pager)
    credits = new Pettanr.Views.Scroll.PlayModule.Credits({parent: this})
    panels = new Pettanr.Views.Scroll.PlayModule.Panels({
      parent: this,
      items: @items,
      credits: credits,
    })
    @listenTo(credits, 'click:credit:icon', @click_credit_icon)
    this.$el.append(panels.render().el)
    this.$el.append(credits.render().el)
    rb = new Tag.RowBreak()
    this.$el.append(rb.render().el)
    # paginate(@pager)
    this
  
  click_credit_icon: (item) ->
    @trigger('http_get', item.show_url())
  
class Pettanr.Views.Scroll.Play extends Backbone.View
  tagName: 'div'
  className: 'show'
  
  initialize: (options) ->
    @item = options.item
    @list = options.list
    icon_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id')})
    caption_url = Pettanr.url(@item.table_name(), 'play', {id: @item.get('id')})
    prof_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id'), format: 'prof'})
    @header = new Pettanr.Views.Scroll.PlayModule.Header({
      item: @item, 
      caption: @item.get('title'), 
      icon_url: icon_url,
      caption_url: caption_url,
      prof_url: prof_url
    })
    @author = new Pettanr.Views.Show.HeaderAuthor({item: @item})
    @body = new Pettanr.Views.Scroll.PlayModule.Body({
      parent: this,
      list: @list, 
    })
    @owner = new Pettanr.Views.Scroll.PlayModule.Owner({item: @item})
    @listenTo(@header, 'click:pick', @click_pick)
    @listenTo(@header, 'click:icon', @click_show)
    @listenTo(@header, 'click:caption', @click_show)
    @listenTo(@header, 'click:prof', @click_prof)
  
  render: () ->
    this.$el.html('')
    this.$el.append(@header.render().el)
    this.$el.append(@author.render().el)
    this.$el.append(@body.render().el)
    this.$el.append(@owner.render().el) if @item.is_own()
    this
  
