# -*- encoding: utf-8 -*-
require 'spec_helper'
#スクロール

describe Author do
  include OwnerMacros
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @user_itojun = FactoryGirl.create :user_itojun_with_owner
    @author_yas =  @user_yas.author
    @artist_yas =  @user_yas.artist
    @author_itojun =  @user_itojun.author
    @artist_itojun =  @user_itojun.artist
    @operator = Operator.new [@user_yas]
    @model = Author
  end
  
  describe 'editに於いて' do
    before do
    end
    
    it 'オペレータが自分を編集しようとしているなら返す' do
      allow_any_instance_of(@model).to receive(:own?).and_return(true)
      expect(@model.edit(@author_yas.id, @operator)).to eq @author_yas
    end
    
    it '他人が変更を加えようとしているときは例外を発生させる' do
      allow_any_instance_of(@model).to receive(:own?).and_return(false)
      expect {@model.edit(@author_yas.id, @operator)}.to raise_error(ActiveRecord::Forbidden)
    end
  end
  
  describe 'overwriteに於いて' do
    before do
    end
    
    it 'ユーザーIDをオペレーターで上書きしている' do
      @author_yas.user_id = nil
      @author_yas.overwrite(@operator)
      expect(@author_yas.user_id).to eq @user_yas.id
    end
  end
  
end

describe Artist do
  include OwnerMacros
  before do
    @admin =FactoryGirl.create :admin
    @user_yas = FactoryGirl.create :user_yas_with_owner
    @user_rom = FactoryGirl.create :user_rom
    @user_itojun = FactoryGirl.create :user_itojun_with_owner
    @author_yas =  @user_yas.author
    @artist_yas =  @user_yas.artist
    @author_itojun =  @user_itojun.author
    @artist_itojun =  @user_itojun.artist
    @operator = Operator.new [@author_yas]
    @model = Artist
  end
  
  describe 'editに於いて' do
    before do
    end
    
    it 'オペレータが自分を編集しようとしているなら返す' do
      allow_any_instance_of(@model).to receive(:own?).and_return(true)
      expect(@model.edit(@artist_yas.id, @operator)).to eq @artist_yas
    end
    
    it '他人が変更を加えようとしているときは例外を発生させる' do
      allow_any_instance_of(@model).to receive(:own?).and_return(false)
      expect {@model.edit(@artist_yas.id, @operator)}.to raise_error(ActiveRecord::Forbidden)
    end
  end
  
  describe 'overwriteに於いて' do
    before do
    end
    
    it 'ユーザーIDをオペレーターで上書きしている' do
      @artist_yas.user_id = nil
      @artist_yas.overwrite(@operator)
      expect(@artist_yas.user_id).to eq @user_yas.id
    end
  end
  
end
